#!/usr/bin/env python

from __future__ import division
from sys import exit
import pygame

class Player(pygame.sprite.Sprite):
    def __init__(self, screen, pos=(0, 0), play_area=(1000, 1000)):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = pygame.Surface((20, 20))
        self.image.fill((0, 100, 255))
        self.rect = self.image.get_rect()
        self.update_screen(screen)
        self.pos = list(pos)
        self.play_area = play_area
        self.offset = [0, 0]
        self.update_pos()
        self.gravity = 20/60    # Speed increases by 10px/s/s
        self.speed = [0, 0]
        self.jump = 0
        self.jump_last = False
    
    def update_pos(self):
        self.clamp_pos()
        left_bound  = self.screen_size[0] // 2 - self.play_area[0] // 2
        right_bound = self.play_area[0] // 2 - self.screen_size[0] // 2
        upper_bound = self.screen_size[1] // 2 - self.play_area[1] // 2
        lower_bound = self.play_area[1] // 2 - self.screen_size[1] // 2
        
        if self.play_area[0] <= self.screen_size[0]:
            self.offset[0] = 0
        else:
            self.offset[0] = -self.pos[0]
            self.offset[0] = max(left_bound, self.offset[0])
            self.offset[0] = min(right_bound, self.offset[0])
        
        if self.play_area[1] <= self.screen_size[1]:
            self.offset[1] = 0
        else:
            self.offset[1] = -self.pos[1]
            self.offset[1] = max(upper_bound, self.offset[1])
            self.offset[1] = min(lower_bound, self.offset[1])
            
        self.rect.centery = self.pos[1] + self.offset[1] + self.screen_size[1] // 2
        self.rect.centerx = self.pos[0] + self.offset[0] + self.screen_size[0] // 2
        
    def clamp_pos(self):
        if self.pos[0] < -self.play_area[0] // 2:
            self.pos[0] = -self.play_area[0] // 2
        elif self.pos[0] > self.play_area[0] // 2:
            self.pos[0] = self.play_area[0] // 2
        
        if self.pos[1] < -self.play_area[1] // 2:
            self.pos[1] = -self.play_area[1] // 2
        elif self.pos[1] > self.play_area[1] // 2:
            self.pos[1] = self.play_area[1] // 2
            
    def clamp_speed(self):
        self.speed[0] = min(self.speed[0], 10)
        self.speed[0] = max(self.speed[0], -10)
        self.speed[1] = min(self.speed[1], 20)
    
    def move(self, dir, collisions, collide_self):
        # print (collide_self, collisions, dir)
        if dir[0] or dir[2]:
            if dir[0] and not dir[2] and self.speed[0] > 0:
                self.speed[0] /= 1.5
            elif dir[2] and not dir[0] and self.speed[0] < 0:
                self.speed[0] /= 1.5
            self.speed[0] -= 0.5*dir[0] - 0.5*dir[2]
        else:
            self.speed[0] /= 1.5
            
        if self.jump_last and not dir[1]:
            self.speed[1] += self.jump / 1.5
        if not dir[1] and not collisions[3]:
            self.jump = 0
            
        if collisions[1] and self.speed[1] < 0 and not (collisions[0] and not collisions[2] or collisions[2] and not collisions[0]):
            self.speed[1] = 0
            self.jump = 0
        if not collisions[3] or (collisions[3] and collisions[1]):
            self.speed[1] += self.gravity
        if ((collisions[3] and dir[1]) and self.jump) or dir[1] and self.jump and not collisions[1]:
            # print ("heyo")
            self.speed[1] = -10
            self.jump -= 1
            # print (self.jump)
        if collisions[3] and self.speed[1] >= 0 and not (collisions[3] and collisions[1]):
            self.speed[1] = 0
            if not dir[1]:
                self.jump = 12
            if collide_self and collisions[3] and not collisions[1]:
                # print ("in")
                self.pos[1] -= self.rect.bottom - collide_self.rect.top
                
        # This case assures that if we jump into an intersection so that \
        #   all four collisions are true and self_collision is also true \
        #   we won't accidentally pass through a wall
        if collisions[0] and collisions[1] and collisions[2] and collisions[3]:
            self.pos[0] -= 2*self.speed[0]
            self.pos[1] -= 2*self.speed[1]
            self.speed[0] = 0
            self.speed[1] = 0
            self.jump = 0
            
            
        # just for reference, this if statement used to look like this
        # if collisions[2] and not collisions[0] and ((collisions[1] and collisions[3]) or not (collisions[1] or collisions[3]) or (collisions[3] and (dir[0] or dir[2])) or collisions[1]):
        # but thanks to this nifty trick involving truth tables and discrete math, I was able to shorten it to this
        if collisions[0] and not collisions[2] and not ((not collisions[1]) and collisions[3] and not (dir[0] or dir[2])):
            self.speed[0] = min(0, self.speed[0])
            if collide_self:
                # print ("in")
                self.pos[0] -= self.rect.right - collide_self.rect.left
        if collisions[2] and not collisions[0] and not ((not collisions[1]) and collisions[3] and not (dir[0] or dir[2])):
            self.speed[0] = max(0, self.speed[0])
            if collide_self:
                # print ("in")
                self.pos[0] -= self.rect.left - collide_self.rect.right
        # self.pos[1] += y
        
        self.clamp_speed()
        self.pos[0] += self.speed[0]
        self.pos[1] += self.speed[1]
        
        self.update_pos()
        self.jump_last = dir[1]
    
    def update_screen(self, screen):
        self.screen_size = screen.get_size()
        
    def get_offset(self):
        return list(self.offset)
        
    def update(self):
        self.update_pos()
        
        
        
class Collision_Box(pygame.sprite.Sprite):
    def __init__(self, size, offset, color=(0, 0, 0)):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = pygame.Surface(size)
        self.image.fill(color)
        self.rect = self.image.get_rect()
        self.offset = offset
        
    def set_pos(self, x, y):
        self.rect.center = (x + self.offset[0], y + self.offset[1])
        
    def collide(self, group):
        return pygame.sprite.spritecollideany(self, group)
        
        
        
# class Pysical_Object(pygame.sprite.Sprite):
    # def set_pos(self, x, y):
        # self.rect.centerx = x + self.offset[0] + self.screen_size[0] // 2
        # self.rect.centery = y + self.offset[1]  + self.screen_size[1] // 2
    
    # def update_screen(self, screen):
        # self.screen_size = screen.get_size()
        
        
        
class Ground(pygame.sprite.Sprite):
    def __init__(self, screen, size, offset):
        print ("aaaaaa")
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = pygame.Surface(size)
        self.image.fill((255, 0, 255))
        self.rect = self.image.get_rect()
        self.rect.center = offset
        self.offset = offset
        # super(Pysical_Object, self).update_screen(screen)
        self.update_screen(screen)
        
    def set_pos(self, x, y):
        self.rect.centerx = x + self.offset[0] + self.screen_size[0] // 2
        self.rect.centery = y + self.offset[1]  + self.screen_size[1] // 2
    
    def update_screen(self, screen):
        self.screen_size = screen.get_size()
        
        
        
class Background(pygame.sprite.Sprite):
    def __init__(self, screen, image, pos=(0, 0)):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = image
        self.rect = self.image.get_rect()
        self.update_screen(screen)
        
    def set_pos(self, x, y):
        self.rect.center = (x + self.screen_size[0] // 2, y + self.screen_size[1] // 2)
    
    def update_screen(self, screen):
        self.screen_size = screen.get_size()
        

def main():
    screen_size = (800, 500)
    play_area = (1000, 1000)
    screen = pygame.display.set_mode(screen_size)
    
    player_group = pygame.sprite.Group()
    background_group = pygame.sprite.Group()
    no_draw_group = pygame.sprite.Group()
    physical_group = pygame.sprite.Group()
    static_group = pygame.sprite.Group()
    
    Player.containers = player_group
    Collision_Box.containers = no_draw_group
    Background.containers = background_group, static_group
    Ground.containers = physical_group, static_group
    
    clock = pygame.time.Clock()
    
    player = Player(screen, (0, -300), (1000, 1000))
    Background(screen, pygame.image.load(r"C:\Users\Cody\Pictures\temp.png"))
    
    collisions = []
    # right / up / left / down
    collisions.append(Collision_Box((1, 20), (10, 0)))
    collisions.append(Collision_Box((20, 1), (0, -11), (0,0,255)))
    collisions.append(Collision_Box((1, 20), (-11, 0), (0,255,0)))
    collisions.append(Collision_Box((20, 1), (0, 10), (255,0,0)))
    
    Ground(screen, (play_area[0], 30), (0, play_area[1] // 2))
    Ground(screen, (30, (play_area[1]-screen_size[1])//2), (0, (play_area[1]-screen_size[1]//2) // 2))
    Ground(screen, (30, 100), (300, (play_area[1]-screen_size[1]//2) // 2))
    Ground(screen, (30, 100), (-300, (play_area[1]-screen_size[1]//2) // 2))
    Ground(screen, (100, 100), (0, 0))
    
    Ground(screen, (200, 30), (0, 50))
    Ground(screen, (screen_size[0], 30), (0, screen_size[1] // 2))
    
    while True:
        # Get all the keypresses
        keys = pygame.key.get_pressed()
        # Move the player
        player.move(
            (
                keys[pygame.K_LEFT], 
                keys[pygame.K_UP], 
                keys[pygame.K_RIGHT], 
                keys[pygame.K_DOWN],
            ), 
            [box.collide(physical_group) for box in collisions] , 
            pygame.sprite.spritecollideany(player, physical_group)
        )
        
        for item in collisions:
            item.set_pos(*player.rect.center)
            
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                exit(0)
        player_group.update()
        offset = player.get_offset()
        
        for item in static_group:
            item.set_pos(*offset)
        
        screen.fill((255, 255, 255))
        background_group.draw(screen)
        physical_group.draw(screen)
        player_group.draw(screen)
        # no_draw_group.draw(screen)
        pygame.display.flip()
        clock.tick(60)

if __name__ == "__main__":
    main()